%let name=tar_heel_traveler;
filename odsout '.';

/*
List of WRAL TV5 Tar Heel Traveler stories.
http://www.wral.com/lifestyles/travel/asset_gallery/3337071/

New location: 
https://www.wral.com/tar-heel-traveler/17382329/?s=9
(need to do the 2019 stories)

I use the following table to look up zip codes (I typically use the first 
zip code listed for the given city)...
http://www.mongabay.com/igapo/zip_codes/NC.htm
*/

%let origin=27606;

/*
In the data below, the first number is a zipcode - not necessarily the "exact"
zipcode of the story, but just a zipcode in the city the story is from (the 
latitude/longitude and city name is looked up in sashelp.zipcode).
The 2nd number is the last part of the web url on WRAL's website, for 
the video.
*/
data raw_data;
infile './tar_heel_traveler.txt' pad truncover;
input mmm $ 1-3 yyyy $ 4-7 dest_zip wralvid story_name $ 24-120;
origin_zip=&origin;
distance=zipcitydistance(&origin,dest_zip);
run;

/* Use the new v9.2 "proc geocode" */
/* Look up the origin's lat/long (this is where all the lines branch out from) */
proc geocode data=raw_data out=raw_data
 addresszipvar=origin_zip nocity;
run;
proc datasets;
 modify raw_data;
 rename x = origin_long;
 rename y = origin_lat;
run;

/* look up the destinations' city (ignoring the lat/long this time) */
proc geocode data=raw_data out=raw_data
 addresszipvar=dest_zip attributevar=(city statecode) nocity;
run;
data raw_data; set raw_data (rename = (statecode=state));
run;
/* now that you have city, do city centroid geocoding */
proc geocode data=raw_data out=raw_data lookup=sashelp.zipcode CITY
 addresszipvar=dest_zip;
run;
proc datasets;
 modify raw_data;
 rename x = dest_long;
 rename y = dest_lat;
 rename state = statecode;
run;

































*options nocenter;
goptions device=png;
goptions xpixels=1000 ypixels=475;
goptions border;



/* **************************************************************************** */
/* first, plot everything (later, do it by year) */
data table_data; set raw_data;
run;

/* 
Count how many stories are in each city, and we also only 
need 1 unique obsn per each city.
*/
proc sql noprint;
create table my_data as
select *, count(*) as city_count 
from table_data 
group by city
order by city;
quit; run;

/* You only need to keep the first obsn for each city, to do the map */
data my_data; set my_data;
 by city;
 if first.city then output;
run;


/*
Sort them so that the farthest-away ones will be drawn first, 
and the closer ones drawn last.  That way, the closer ones 
will be "on top" of the farther ones' lines (in the cases 
where the lines & markers overlap.
*/
proc sort data=my_data out=my_data;
by descending distance;
run;

quit; run;

data my_anno; set my_data;
length function $8 color $12;
anno_flag=1;
xsys='2'; ysys='2'; hsys='3'; when='a';

long=origin_long;
lat=origin_lat;
function='move'; output;

long=dest_long;
lat=dest_lat;
function='draw'; size=.001; color='CXff0000'; output;

function='pie'; rotate=360; size=1.3;
color='white'; style='psolid'; output;
color='cx1b2e7d'; style='pempty'; href=''; output;

run;

data my_anno; set my_anno;
 if long^=. and lat^=. then output;
run;




data my_map; set mapsgfk.us_counties (where=((density<=3) and (statecode='NC')));
run;

/* combine the map & markers, project it, then separate them again */
data combined; set my_map my_anno; 
run;
proc gproject data=combined out=combined latlong eastlong degrees dupok project=hammer;
id state county;
run;
data my_map my_anno; set combined;
if anno_flag=1 then output my_anno;
else output my_map;
run;


/* do this *after* the gproject, or the long char variable will slow down the gproject */
data my_anno; set my_anno;
  length html $500;

/* only give the drilldown to the outer/empty circle of the dots */
if style='pempty' then do;
  /* if there's only 1 story in this city, link directly to it */
  if city_count = 1 then do;
   html=
    ' title='||quote(trim(left(city))||': '||'0d'x||trim(left(story_name)))||
    'target="tht2"'||
    ' href="http://www.wral.com/lifestyles/outdoors/video/'||trim(left(wralvid))||'"';
   end;
  /* if there's more than 1 story, link to the html 'anchor' for a list of stories in that city */
  else do;
   html='title='||quote(trim(left(city))||': '||'0d'x||trim(left(city_count))||' stories')||
    ' href="#'||translate(trim(left(city)),'_',' ')||'"';
   end;
 end;

run;



/* annotate shadow behind map */
data shadow_anno; set my_map; 
by county segment notsorted;
length function color $8;    
xsys='2'; ysys='2'; when='B';
color="cx013a93"; size=1.75; style='msolid';
if first.county or first.segment then function='poly';
else function='polycont';
run;

/* slight offset, for shadow effect */
data shadow_anno; set shadow_anno;
x=x+1; y=y-1;
run;


data all_anno;  
 set shadow_anno my_anno;
run;


data anno_logos;
length function $8 style $35 html $300 imgpath $300;
ysys='3'; xsys='3'; hsys='3';

/* invisible box around the SAS logo, with drilldown link to SAS analytics page */
html='title='||quote('Analytics powered by SAS')||
 ' href='||quote('http://www.sas.com/en_us/software/analytics.html');
when='b';
function='move'; x=41; y=3; output;
function='box'; x=x+17.6; y=y+9.3; output;
html='';

/* Blue background, with SAS Analytics logo (background) */
when='b';
function='move'; x=0; y=0; output;
function='image'; x=100; y=100; imgpath='./TarHeelTraveler-ripple-SASlogo.png'; style='tile'; output;

/* Tar Heel Traveler logo (foreground) */
when='a';
function='move'; x=7; y=5; output;
function='image'; x=x+18; y=y+29; imgpath='./tht-logo-180x135.png'; style='fit'; output;

run;



/* Create the year labels w/ drilldown links, for the main/all map */
proc sql noprint;
create table anno_years as
select unique yyyy, yyyy as text from raw_data
order by yyyy;
quit; run;
data anno_years; set anno_years;
function='label'; position='5'; style='albany amt/bold'; color='white'; size=3.2;
xsys='3'; ysys='3'; hsys='3'; when='a';
x=85;
y=30.5-(_n_*3.9);
if yyyy>2012 then do;
 x=x+5;
 y=y+(3.9*6);
 end;
if yyyy>2018 then do;
 x=x+5;
 y=y+(3.9*6);
 end;
length html $300;
html=
 'title='||quote(trim(left(text))||' stories')||
 ' href='||quote('tar_heel_traveler_'||trim(left(text))||'.htm');
run;

data anno_logos_plus; set anno_logos anno_years;
run;


/* some font & color changes requested by Tarheel Traveler web admin (Kelly Gardner) */
/*
proc template;
 define style Styles.tht_style;
  parent = styles.htmlblue;
  replace Output from Container / bordercolor=#eeeeee;
  style fonts / 'docFont'=(" Helvetica, Arial ",2);
  style colors / 'docbg'=white;
 end;
run;
*/

 
ODS LISTING CLOSE;
ODS HTML path=odsout body="&name..htm" 
 (title="Tar Heel Traveler stories") 
 /*style=tht_style*/ style=htmlblue;

pattern1 color=cxe8d6cc v=solid;

goptions gunit=pct htitle=6 htext=6 ftitle="albany amt/bold" ftext="albany amt/bold" ctext=grayee;

title1 link="http://www.wral.com/lifestyles/travel/asset_gallery/3337071/"
    ls=3.0 "Click Destinations to see Videos!";

title2 a=90 h=.5pct " ";
title3 a=-90 h=.5pct " ";

footnote1 h=8 " ";


proc gmap map=my_map data=my_map anno=all_anno;
id state county; 
choro state / levels=1 nolegend 
 coutline=cx968d86 coutline=graybb  
 anno=anno_logos_plus
 des='' name="&name";
run;

quit;
ODS HTML CLOSE;
ODS LISTING;
 


/*
For the table with links, I'm scripting out my own html.
I could easily use "proc print" to do a table with links,
but I'm wanting to stick a lot of space between the sections
of the table, and put in my own anchors at a very specific
location in the white-space ... just a personal preference
and look-n-feel and usability thing for this particular
example.
dest_zip wralvid story_name
*/


proc sort data=table_data out=table_data;
 by city dest_zip story_name;
run;

data table_data; set table_data;
 length link $300;
 link='http://www.wral.com/lifestyles/outdoors/video/'||trim(left(wralvid));
run;

data table_data; set table_data;
 length link $300 href $300 anchor $100;
 /* The google query doesn't like '&', so I'm changing those to spaces, in the href */
 href = 'href="' || trim(left(translate(link,' ','&'))) || '"';
 link = '<a ' || trim(href) || ' target="body">' || htmlencode(trim(story_name)) || '</a>';
 anchor = '<a name="' || translate(trim(left(city)),'_',' ') || '"></a>';
run;


filename mytable "./&name..htm" mod;
data _null_; set table_data end=last;
 by city;
 file mytable;

 if first.city then do;
  put "</br>";
  put "</br>";
  put "</br>";
  put anchor;
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "<table align='center' border='1' cellpadding='4' cellspacing='0' bordercolor='eeeeee'>";
  put "<th width='185px'>";
  put "City</th>";
  put "<th width='500px'>Click Link to see Story</th>";
 end;
 put "<tr>";
 put "<td align='center'>";
 put city;
 put "</td>";
 put "<td>";
 put link;
 put "</td>";
 put "</tr>";
 if last.city then do;
  put "</table>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</body>";
  put "</html>";
 end;

run;
quit;


/* **************************************************************** */
/* now a macro to do it by year */
 
%macro do_year(year);

ODS LISTING CLOSE;
ODS HTML path=odsout body="&name._&year..htm" 
 (title="WRAL TV5 - Tar Heel Traveler &year stories (SAS Map)") 
 /*style=tht_style*/;

data table_data; set raw_data (where=(yyyy="&year"));
run;

/* 
Count how many stories are in each city, and we also only 
need 1 unique obsn per each city.
*/
proc sql noprint;
create table my_data as
select *, count(*) as city_count 
 from table_data 
 group by city
 order by city;
quit; run;

/* You only need to keep the first obsn for each city, to do the map */
data my_data; set my_data;
 by city;
 if first.city then output;
run;


/*
Sort them so that the farthest-away ones will be drawn first, 
and the closer ones drawn last.  That way, the closer ones 
will be "on top" of the farther ones' lines (in the cases 
where the lines & markers overlap.
*/
proc sort data=my_data out=my_data;
by descending distance;
run;

quit; run;

data my_anno; set my_data;
length function $8 color $12;
anno_flag=1;
xsys='2'; ysys='2'; hsys='3'; when='a';

long=origin_long;
lat=origin_lat;
function='move'; output;

long=dest_long;
lat=dest_lat;
function='draw'; size=.001; color='CXff0000'; output;

function='pie'; rotate=360; size=1.3;
color='white'; style='psolid'; output;
color='cx1b2e7d'; style='pempty'; href=''; output;

run;

data my_anno; set my_anno;
 if long^=. and lat^=. then output my_anno;
run;




data my_map; set mapsgfk.us_counties (where=((density<=3) and (statecode='NC')));
run;

/* combine the map & markers, project it, then separate them again */
data combined; set my_map my_anno; 
run;
proc gproject data=combined out=combined latlong eastlong degrees dupok project=hammer;
id state county;
run;
data my_map my_anno; set combined;
if anno_flag=1 then output my_anno;
else output my_map;
run;


/* do this *after* the gproject, or the long char variable will slow down the gproject */
data my_anno; set my_anno;
  length html $500;

/* only give the drilldown to the outer/empty circle of the dots */
if style='pempty' then do;
  /* if there's only 1 story in this city, link directly to it */
  if city_count = 1 then do;
   html='title='||quote(trim(left(city))||': '||'0d'x||trim(left(story_name)))||
    'target="tht2"'||
    ' href="http://www.wral.com/lifestyles/outdoors/video/'||trim(left(wralvid))||'"';
   end;
  /* if there's more than 1 story, link to the html 'anchor' for a list of stories in that city */
  else do;
   html='title='||quote(trim(left(city))||': '||'0d'x||trim(left(city_count))||' stories')||
    ' href="#'||translate(trim(left(city)),'_',' ')||'"';
   end;
 end;

run;



/* annotate shadow behind map */
data shadow_anno; set my_map; 
by county segment notsorted;
length function color $8;    
xsys='2'; ysys='2'; when='B';
color="cx013a93"; size=1.75; style='msolid';
if first.county or first.segment then function='poly';
else function='polycont';
run;

/* slight offset, for shadow effect */
data shadow_anno; set shadow_anno;
x=x+1; y=y-1;
run;

data all_anno;  
 set shadow_anno my_anno;
run;


proc gmap map=my_map data=my_map anno=all_anno;
note move=(77,7) height=20 "&year";
id state county; 
choro state / levels=1 nolegend 
 coutline=cx968d86 coutline=graybb  
 anno=anno_logos
 des='' name="&name._&year";
run;

proc print data=no_lat_long; run;

quit;
ODS HTML CLOSE;
ODS LISTING;
 


/*
For the table with links, I'm scripting out my own html.
I could easily use "proc print" to do a table with links,
but I'm wanting to stick a lot of space between the sections
of the table, and put in my own anchors at a very specific
location in the white-space ... just a personal preference
and look-n-feel and usability thing for this particular
example.
dest_zip wralvid story_name
*/


proc sort data=table_data out=table_data_year;
 by city dest_zip story_name;
run;

data table_data_year; set table_data_year;
 length link $300;
 link='http://www.wral.com/lifestyles/outdoors/video/'||trim(left(wralvid));
run;

data table_data_year; set table_data_year;
 length href $300 anchor $100;
 /* The google query doesn't like '&', so I'm changing those to spaces, in the href */
 href = 'href="' || trim(left(translate(link,' ','&'))) || '"';
/* hmm ... if I control the color of the links here, then vlink (color of visited link) no longer works. */
/*
 link = '<a ' || trim(href) || ' target="tht2"> <font color="0F3569">' || htmlencode(trim(story_name)) || '</font></a>';
*/
 link = '<a ' || trim(href) || ' target="tht2">' || htmlencode(trim(story_name)) || '</a>';
 anchor = '<a name="' || translate(trim(left(city)),'_',' ') || '"></a>';
run;


filename mytable "./&name._&year..htm" mod;
data _null_; set table_data_year end=last;
 by city;
 file mytable;

 if first.city then do;
  put "</br>";
  put "</br>";
  put "</br>";
  put anchor;
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "<table align='center' border='1' cellpadding='4' cellspacing='0' bordercolor='eeeeee'>";
  put "<th width='185px'>";
  put "City</th>";
  put "<th width='500px' align='left'>Click Link to see Story</th>";
 end;
 put "<tr>";
 put "<td align='center'>";
 put city;
 put "</td>";
 put "<td>";
 put link;
 put "</td>";
 put "</tr>";
 if last.city then do;
  put "</table>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</br>";
  put "</body>";
  put "</html>";
 end;

run;
quit;

%mend;

/*
*/
%do_year(2007);
%do_year(2008);
%do_year(2009);
%do_year(2010);
%do_year(2011);
%do_year(2012);
%do_year(2013);
%do_year(2014);
%do_year(2015);
%do_year(2016);
%do_year(2017);
%do_year(2018);
%do_year(2019);
